@@ -3,7 +3,9 @@
  * CommandChannel routines
  */
 #include "config.h"
+
 #ifdef HAVE_COMMAND
+
 #include <stdio.h>
 #include <stdlib.h>
 #include <stdbool.h>
@@ -18,7 +20,7 @@
 #endif
 #include "command.h"
 #include "libstr.h"
-#include "child.h"
+#include "client.h"
 #include "session.h"
 
 #define MAX_IDLE_TIME 30
@@ -32,9 +34,9 @@ pthread_mutex_t command_mutex;
 
 unsigned long clients_served = 0, req_files = 0, req_cgi = 0, req_index = 0;
 
-/* A child needs to change a session record. The CommandChannel must stay out.
+/* A client needs to change a session record. Admins must stay out.
  */
-void tell_cc_Im_busy(void) {
+void CC_client_lock(void) {
 	pthread_mutex_lock(&command_mutex);
 
 	while (cl_lock < 0) {
@@ -45,15 +47,15 @@ void tell_cc_Im_busy(void) {
 	pthread_mutex_unlock(&command_mutex);
 }
 
-/* Hiawatha is done changing the session record.
+/* Client is done changing the session record.
  */
-void tell_cc_Im_done(void) {
+void CC_client_unlock(void) {
 	cl_lock--;
 }
 
-/* CommandChannel wants to read a session record. Childs must stay out.
+/* Admin wants to read a session record. nr_of_clients must stay out.
  */
-void tell_children_Im_busy(void) {
+void CC_admin_lock(void) {
 	pthread_mutex_lock(&command_mutex);
 
 	while (cl_lock > 0) {
@@ -64,9 +66,9 @@ void tell_children_Im_busy(void) {
 	pthread_mutex_unlock(&command_mutex);
 }
 
-/* CommandChannel is done reading a session record.
+/* Admin is done reading a session record.
  */
-void tell_children_Im_done(void) {
+void CC_admin_unlock(void) {
 	cl_lock++;
 }
 
@@ -201,7 +203,6 @@ static void show_help(FILE *fp) {
 				"kickid <client-id> : disconnect client <client-id>\n"
 				"kickip <ip-address>: disconnect clients with that ip-address\n"
 				"kickall            : disconnect all clients\n"
-				"reload             : reload configuration\n"
 				"shutdown           : shutdown the webserver\n"
 				"status             : status of the webserver\n"
 				"quit               : quit CommandChannel\n"
@@ -239,7 +240,9 @@ int handle_admin(t_admin *admin, t_config *config) {
 							ban_ip(ip, time, false);
 						}
 					} else if (strcmp(cmd, "unban") == 0) {
-						if (parse_ip(param, &ip) != -1) {
+						if (strcmp(param, "all") == 0) {
+							retval = cc_UNBAN_CLIENTS;
+						} else if (parse_ip(param, &ip) != -1) {
 							unban_ip(ip);
 						}
 					} else {
@@ -254,17 +257,15 @@ int handle_admin(t_admin *admin, t_config *config) {
 					} else if (strcmp(cmd, "shutdown") == 0) {
 						retval = cc_SHUTDOWN;
 					} else if (strcmp(cmd, "clientlist") == 0) {
-						tell_children_Im_busy();
-						print_childlist(admin->fp);
-						tell_children_Im_done();
+						CC_admin_lock();
+						print_clientlist(admin->fp);
+						CC_admin_unlock();
 					} else if (strcmp(cmd, "banlist") == 0) {
 						print_banlist(admin->fp);
 					} else if (strcmp(cmd, "kickall") == 0) {
 						disconnect_clients(config);
-					} else if (strcmp(cmd, "reload") == 0) {
-						retval = cc_RELOADCONF;
 					} else if (strcmp(cmd, "status") == 0) {
-						fprintf(admin->fp, "Clients served   : %lu\n", clients_served);
+						fprintf(admin->fp, "nr_of_clients served   : %lu\n", clients_served);
 						fprintf(admin->fp, "Files requested  : %lu\n", req_files);
 						fprintf(admin->fp, "CGI's requested  : %lu\n", req_cgi);
 						fprintf(admin->fp, "Indexes requested: %lu\n", req_index);
@@ -273,10 +274,10 @@ int handle_admin(t_admin *admin, t_config *config) {
 					}
 				}
 			} else {
-				memcpy(salt, config->CommandPassword, 2);
+				memcpy(salt, config->command_port->name, 2);
 				salt[2] = '\0';
 				encrypted = crypt(remove_spaces(line), salt);
-				if ((admin->allowed = (strcmp(encrypted, config->CommandPassword) == 0)) == false) {
+				if ((admin->allowed = (strcmp(encrypted, config->command_port->name) == 0)) == false) {
 					retval = cc_DISCONNECT;
 				}
 			}
@@ -296,4 +297,5 @@ int handle_admin(t_admin *admin, t_config *config) {
 
 	return retval;
 }
+
 #endif
