@@ -1,13 +1,14 @@
 #ifndef _COMMAND_H
 #define _COMMAND_H
 
+#ifdef HAVE_COMMAND
+
 #include "config.h"
 #include "serverconfig.h"
 
-#ifdef HAVE_COMMAND
-#define cc_DISCONNECT -1
-#define cc_SHUTDOWN   -2
-#define cc_RELOADCONF  1
+#define cc_DISCONNECT    -1
+#define cc_SHUTDOWN      -2
+#define cc_UNBAN_CLIENTS  1
 
 typedef struct type_admin {
 	int socket;
@@ -18,10 +19,10 @@ typedef struct type_admin {
 	struct type_admin *next;
 } t_admin;
 
-void tell_cc_Im_busy(void);
-void tell_cc_Im_done(void);
-void tell_children_Im_busy(void);
-void tell_children_Im_done(void);
+void CC_client_lock(void);
+void CC_client_unlock(void);
+void CC_admin_lock(void);
+void CC_admin_unlock(void);
 
 void init_commandmodule(void);
 int add_admin(int sock);
@@ -31,6 +32,7 @@ t_admin *first_admin(void);
 t_admin *next_admin(void);
 int handle_admin(t_admin *admin, t_config *config);
 void disconnect_admins(void);
+
 #endif
 
 #endif
