@@ -1,7 +1,5 @@
 #include "config.h"
-#include <stdio.h>
 #include <stdlib.h>
-#include <stdbool.h>
 #include <unistd.h>
 #include <string.h>
 #include <time.h>
@@ -28,99 +26,71 @@ long filesize(char *filename) {
 	}
 }
 
-/* Check the accessrights of a file. (internal use)
- */
-static enum t_extbool rights_oke(struct stat *status, enum t_filetype filetype, mode_t read, mode_t exec) {
-	switch (filetype) {
-		case file:
-			if ((status->st_mode & read) == read) {
-				return yes;
-			}
-			break;
-		case path_to:
-			if ((status->st_mode & exec) == exec) {
-				return yes;
-			}
-			break;
-		case executable:
-		case directory:
-			if (((status->st_mode & read) == read) && ((status->st_mode & exec) == exec)) {
-				return yes;
-			}
-			break;
-	}
+static enum t_fsbool outside_webroot(char *symlink, char *webroot) {
+	char filename[257], *slash;
+	int size, count;
 
-	return no;
-}
-
-/* Check the accessrights of a file.
- */
-enum t_extbool fs_access_allowed(char *filename, uid_t uid, gid_t gid, t_groups *groups, enum t_filetype filetype) {
-	enum t_extbool allowed = no;
-	bool in_group = false;
-	struct stat status;
-	char *slash;
-	int num = 0;
-	gid_t *group;
-
-	if (stat(filename, &status) == 0) {
-		if (status.st_uid == uid) {
-			/* Check user
-			 */
-			allowed = rights_oke(&status, filetype, S_IRUSR, S_IXUSR);
-		} else {
-			/* Check group
-			 */
-			if (status.st_gid == gid) {
-				allowed = rights_oke(&status, filetype, S_IRGRP, S_IXGRP);
-				in_group = true;
-			} else if (groups != NULL) {
-				group = groups->array;
-				while (num < groups->number) {
-					if (status.st_gid == *group) {
-						allowed = rights_oke(&status, filetype, S_IRGRP, S_IXGRP);
-						in_group = true;
-						break;
-					}
-					group++;
-					num++;
+	if ((size = readlink(symlink, filename, 256)) > 0) {
+		filename[size] = '\0';
+		if (strchr(filename, '/') == NULL) {
+			return no;
+		}
+		if (filename[0] == '/') {
+			// Symlink with complete path
+			if (strncmp(webroot, filename, strlen(webroot)) == 0) {
+				return no;
+			}
+		} else if (strncmp(filename, "../", 3) == 0) {
+			// Symlink that starts wih ../
+			count = 0;
+			while (strncmp(filename + (3 * count), "../", 3) == 0) {
+				count++;
+			}
+			slash = symlink + strlen(symlink);
+			while (count-- > 0) {
+				while ((slash > symlink) && (*slash != '/')) {
+					slash--;
+				}
+				if (slash == symlink) {
+					break;
+				} else {
+					slash--;
 				}
 			}
-
-			/* Check others
-			 */
-			if (in_group == false) {
-				allowed = rights_oke(&status, filetype, S_IROTH, S_IXOTH);
+			if ((size_t)(slash - symlink) >= strlen(webroot)) {
+				return no;
 			}
 		}
-
-		if (allowed == yes) {
-			slash = filename + strlen(filename);
-			while ((slash != filename) && (*slash != '/')) {
-				slash--;
-			}
-			if (slash != filename) {
-				*slash = '\0';
-				allowed = fs_access_allowed(filename, uid, gid, groups, path_to);
-				*slash = '/';
-			}
-		}
-	} else {
-		allowed = error;
+	} else switch (errno) {
+		case EACCES:
+			return no_access;
+		case ENOENT:
+			return not_found;
+		default:
+			return error;
 	}
 
-	return allowed;
+	return yes;
 }
 
-enum t_extbool contains_symlink(char *filename) {
-	enum t_extbool contains = no;
+enum t_fsbool contains_not_allowed_symlink(char *filename, char *webroot) {
+	enum t_fsbool contains = no, outside;
 	struct stat status;
 	char *slash;
 
 	if (lstat(filename, &status) == -1) {
-		return error;
+		switch (errno) {
+			case EACCES:
+				return no_access;
+			case ENOENT:
+				return not_found;
+			default:
+				return error;
+		}
 	} else if (((status.st_mode & S_IFMT) == S_IFLNK) && (status.st_uid != 0)) {
-		return yes;
+		if ((outside = outside_webroot(filename, webroot)) != no) {
+			return outside;
+		}
 	}
 
 	slash = filename + strlen(filename);
@@ -129,7 +99,7 @@ enum t_extbool contains_symlink(char *filename) {
 	}
 	if (slash != filename) {
 		*slash = '\0';
-		contains = contains_symlink(filename);
+		contains = contains_not_allowed_symlink(filename, webroot);
 		*slash = '/';
 	}
 
@@ -138,26 +108,81 @@ enum t_extbool contains_symlink(char *filename) {
 
 /* Check whether a file is directory or not.
  */
-enum t_isdir is_directory(char *file) {
+enum t_fsbool is_directory(char *file) {
 	DIR *dp;
 		
 	if ((dp = opendir(file)) != NULL) {
 		closedir(dp);
 
-		return is_dir;
+		return yes;
 	} else switch (errno) {
 		case EACCES:
 			return no_access;
 		case ENOENT:
 			return not_found;
 		default:
-			return no_dir;
+			return no;
+	}
+}
+
+/* Check whether a file can be executed or not.
+ */
+enum t_fsbool can_execute(char *file, uid_t uid, gid_t gid, t_groups *groups) {
+	struct stat status;
+	gid_t *group;
+	int num = 0;
+
+	if (stat(file, &status) == 0) {
+		if (status.st_uid == uid) {
+			// Check user
+			if ((status.st_mode & S_IXUSR) == S_IXUSR) {
+				return yes;
+			} else {
+				return no;
+			}
+		} else {
+			// Check group
+			if (status.st_gid == gid) {
+				if ((status.st_mode & S_IXGRP) == S_IXGRP) {
+					return yes;
+				} else {
+					return no;
+				}
+			} else if (groups != NULL) {
+				group = groups->array;
+				while (num < groups->number) {
+					if (status.st_gid == *group) {
+						if ((status.st_mode & S_IXGRP) == S_IXGRP) {
+							return yes;
+						} else {
+							return no;
+						}
+					}
+					group++;
+					num++;
+				}
+			}
+
+			// Check others
+			if ((status.st_mode & S_IXOTH) == S_IXOTH) {
+				return yes;
+			} else {
+				return no;
+			}
+		}
+	} else switch (errno) {
+		case EACCES:
+			return no_access;
+		case ENOENT:
+			return not_found;
+		default:
+			return error;
 	}
 }
 
 /* Create a file with the right ownership and accessrights.
  */
-void touch_logfile(char *dir, char *file, uid_t userid, gid_t groupid, mode_t mode) {
+void touch_logfile(char *dir, char *file, mode_t mode, uid_t uid, gid_t gid) {
 	int fd; 
 	char *logfile;
 
@@ -172,16 +197,19 @@ void touch_logfile(char *dir, char *file, uid_t userid, gid_t groupid, mode_t mo
 	if (logfile != NULL) {
 		if ((fd = open(logfile, O_RDONLY)) == -1) {
 			if ((fd = open(logfile, O_CREAT|O_APPEND)) != -1) {
-				fchown(fd, userid, groupid);
 				fchmod(fd, mode);
+				if (getuid() == 0) {
+					fchown(fd, uid, gid);
+				}
 				close(fd);
 			}   
 		} else {
 			close(fd);
 		}   
-	}   
-	if (dir != NULL) {
-		free(logfile);
+
+		if (dir != NULL) {
+			free(logfile);
+		}
 	}   
 }
 
@@ -307,3 +335,97 @@ FILE *fopen_neighbour(char *filename, char *mode, char *neighbour) {
 
 	return fp;
 }
+
+//-----< filelist functions >--------------------------------------------------
+
+/* Read a directory and place the filenames in a list.
+ */
+t_filelist *read_filelist(char *directory) {
+	DIR *dp;
+	t_filelist *filelist = NULL, *file;
+	char filename[257];
+	int dir_len;
+	struct stat status;
+	struct dirent *dir_info;
+
+	if ((dp = opendir(directory)) == NULL) {
+		return NULL;
+	}
+
+	strncpy(filename, directory, 256);
+	filename[256] = '\0';
+	dir_len = strlen(directory);
+
+	while ((dir_info = readdir(dp)) != NULL) {
+		if ((dir_info->d_name[0] != '.') || (strcmp(dir_info->d_name, "..") == 0)) {
+			strcpy(filename + dir_len, dir_info->d_name);
+			stat(filename, &status);
+
+			if ((file = (t_filelist*)malloc(sizeof(t_filelist))) != NULL) {
+				file->name = strdup(dir_info->d_name);
+				file->size = status.st_size;
+				file->time = status.st_mtime;
+				file->is_dir = S_ISDIR(status.st_mode);
+				file->next = filelist;
+			} else {
+				filelist = remove_filelist(filelist);
+				break;
+			}
+			filelist = file;
+		}
+	}
+	closedir(dp);
+
+	return filelist;
+}
+
+/* Sort a list of filenames alfabeticly.
+ */
+t_filelist *sort_filelist(t_filelist *filelist) {
+	t_filelist *start = NULL, *newpos, *prev, *newitem;
+
+	while (filelist != NULL) {
+		newitem = filelist;
+		filelist = filelist->next;
+
+		prev = NULL;
+		newpos = start;
+		while (newpos != NULL) {
+			if (newitem->is_dir && (newpos->is_dir == false)) {
+				break;
+			}
+			if (newitem->is_dir == newpos->is_dir) {
+				if (strcmp(newpos->name, newitem->name) >= 0) {
+					break;
+				}
+			}
+			prev = newpos;
+			newpos = newpos->next;
+		}
+
+		if (prev == NULL) {
+			newitem->next = start;
+			start = newitem;
+		} else {
+			prev->next = newitem;
+			newitem->next = newpos;
+		}
+	}
+	
+	return start;
+}
+
+/* free() a list of filenames.
+ */
+t_filelist *remove_filelist(t_filelist *filelist) {
+	t_filelist *file;
+
+	while (filelist != NULL) {
+		file = filelist;
+		filelist = filelist->next;
+		free(file->name);
+		free(file);
+	}
+
+	return NULL;
+}
