@@ -1,7 +1,6 @@
 #include "config.h"
 #include <stdio.h>
 #include <stdlib.h>
-#include <stdbool.h>
 #include <string.h>
 #include "libstr.h"
 #include "liblist.h"
@@ -56,7 +55,7 @@ int parse_ip(char *line, unsigned long *l_ip) {
 	return 0;
 }
 
-// --< charlist >-----------------------------------------------------------
+// --< charlist >--------------------------------------------------------------
 
 void init_charlist(t_charlist *list) {
 	list->size = 0;
@@ -107,99 +106,7 @@ bool in_charlist(char *item, t_charlist *list) {
 	return false;
 }
 
-void remove_charlist(t_charlist *list) {
-	while (list->size-- > 0) {
-		free(*(list->item + list->size));
-	}
-	free(list->item);
-	list->item = NULL;
-}
-
-// --< bindlist >-----------------------------------------------------------
-
-/* Parse an list of bindings.
- */
-t_bindlist *parse_bindlist(char *rest, int default_port, t_bindlist *list) {
-	t_bindlist *new;
-	char *addr, *port, *name;
-	bool error = false;
-
-	new = list;
-	if (list != NULL) {
-		while (new->next != NULL) {
-			new = new->next;
-		}
-	}
-
-	while (rest != NULL) {
-		split_string(rest, &addr, &rest, ',');
-		if (list == NULL) {
-			if ((list = new = (t_bindlist*)malloc(sizeof(t_bindlist))) == NULL) {
-				error = true;
-				break;
-			}
-		} else {
-			if ((new->next = (t_bindlist*)malloc(sizeof(t_bindlist))) == NULL) {
-				error = true;
-				break;
-			}
-			new = new->next;
-		}
-		new->next = NULL;
-
-		if (split_string(addr, &addr, &name, '-') == 0) {
-			new->name = strdup(name);
-		} else {
-			new->name = NULL;
-		}
-
-		if (split_string(addr, &addr, &port, ':') == 0) {
-			if (parse_ip(addr, &new->ip) == -1) {
-				error = true;
-				break;
-			} else if ((new->port = str2int(port)) == -1) {
-				error = true;
-				break;
-			}
-			new->socket = -1;
-		} else {
-			if (parse_ip(addr, &new->ip) == -1) {
-				// Must be port
-				if ((new->port = str2int(addr)) == -1) {
-					error = true;
-					break;
-				}
-				new->ip = 0;
-			} else {
-				// Is IP address
-				new->port = default_port;
-			}
-		}
-
-	}
-	if (error) {
-		list = remove_bindlist(list);
-	}
-
-	return list;
-}
-
-/* Remove a bindlist.
- */
-t_bindlist *remove_bindlist(t_bindlist *list) {
-	t_bindlist *item;
-
-	while (list != NULL) {
-		item = list;
-		list = list->next;
-		sfree(item->name);
-		free(item);
-	}
-
-	return NULL;
-}
-
-// --< accesslist >---------------------------------------------------------
+// --< accesslist >------------------------------------------------------------
 
 /* Parse an list of access levels.
  */
@@ -308,6 +215,10 @@ t_access ip_allowed(unsigned long ip, t_accesslist *list) {
 	return unknown;
 }
 
+// --< key/valuelist >---------------------------------------------------------
+
+/* Parse an list of key/value combinations.
+ */
 t_keyvalue *parse_keyvaluelist(char *line, t_keyvalue *kvlist, char seperator) {
 	char *key, *value, *rest;
 	t_keyvalue *prev;
