@@ -1,7 +1,8 @@
 #include "config.h"
+
 #ifdef HAVE_SSL
+
 #include <stdio.h>
-#include <stdbool.h>
 #include <string.h>
 #include <assert.h>
 #include <pthread.h>
@@ -11,7 +12,6 @@ static pthread_mutex_t *locks;
 static int lockCount;
 /* Global SSL structures */
 static BIO *stderrBio;
-static SSL_CTX *sslContext;
 
 /* SSL multithread locking callback */
 static void locking_callback(int mode, int n, const char *file, int line) {
@@ -29,8 +29,22 @@ static unsigned long id_callback() {
 	return (unsigned long)pthread_self();
 }
 
-bool sslInit(const char* sslKeyFile, const char* sslPassword) {
-	SSL_METHOD *meth;
+static int sslPasswordCB(char *passwordBuffer, int size, int rwflag, void *data) {
+	char *keyfilePassword;
+	int pwLength;
+
+	keyfilePassword = (char*)data;
+	pwLength = strlen(keyfilePassword);
+
+	if (size < pwLength) {
+  		return 0;
+	}
+	strncpy(passwordBuffer, keyfilePassword, pwLength);
+
+	return pwLength;
+} 
+
+void sslInit(void) {
 	int i;
 
 	/* Load the SSL algorithms and error strings (global)..*/
@@ -39,31 +53,6 @@ bool sslInit(const char* sslKeyFile, const char* sslPassword) {
 
 	/* Wrap stderr in a BIO (an openssl IO abstraction object)..*/
 	stderrBio = BIO_new_fp(stderr, BIO_NOCLOSE);
-  
-	/* Create the SSL context (the context is used to manage keyfiles, 
-	 * CA stuff etc. so they have to be loaded only once). */
-	//meth = TLSv1_method();
-	meth = SSLv23_method();
-	sslContext = SSL_CTX_new(meth);
-
-	/* Don't allow SSLv2 */
-	SSL_CTX_set_options(sslContext, SSL_OP_NO_SSLv2);
-
-	/* Load certificate from keyfile..*/
-	if (!SSL_CTX_use_certificate_chain_file(sslContext, sslKeyFile)) {
-		fprintf(stderr, "Failed to read SSL Certificate file.\n");
-		ERR_print_errors(stderrBio);
-		return false;
-	}
-
-	/* Load key from keyfile..*/
-	SSL_CTX_set_default_passwd_cb(sslContext, sslPasswordCB);
-	SSL_CTX_set_default_passwd_cb_userdata(sslContext, (void*)sslPassword);
-	if (!SSL_CTX_use_PrivateKey_file(sslContext, sslKeyFile, SSL_FILETYPE_PEM)) {
-		fprintf(stderr, "Failed to read SSL key file.\n");
-		ERR_print_errors(stderrBio);
-		return false;
-	}
 
 	/* Initialize SSL library's thread locks. */
 	lockCount = CRYPTO_num_locks();
@@ -79,39 +68,41 @@ bool sslInit(const char* sslKeyFile, const char* sslPassword) {
   
 	/* Register thread id callback. */
 	CRYPTO_set_id_callback(id_callback);
- 
-	return true;
 }
 
-void sslCleanup(void) {
-	int i;
+SSL_CTX *ssl_binding(const char* sslKeyFile, const char* sslPassword) {
+	SSL_METHOD *meth;
+	SSL_CTX    *sslContext;
 
-	/* Free library resources */
-	SSL_CTX_free(sslContext);
-	BIO_free(stderrBio);
-  
-	/* Destroy locks. */
-	for (i = 0; i < lockCount; i++) {
-		pthread_mutex_destroy(&locks[i]);
+	/* Create the SSL context (the context is used to manage keyfiles, 
+	 * CA stuff etc, so they have to be loaded only once). */
+	//meth = TLSv1_method();
+	meth = SSLv23_method();
+	sslContext = SSL_CTX_new(meth);
+
+	/* Don't allow SSLv2 */
+	SSL_CTX_set_options(sslContext, SSL_OP_NO_SSLv2);
+
+	/* Load certificate from keyfile..*/
+	if (!SSL_CTX_use_certificate_chain_file(sslContext, sslKeyFile)) {
+		fprintf(stderr, "Failed to read SSL Certificate file.\n");
+		ERR_print_errors(stderrBio);
+		return NULL;
 	}
+
+	/* Load key from keyfile..*/
+	SSL_CTX_set_default_passwd_cb(sslContext, sslPasswordCB);
+	SSL_CTX_set_default_passwd_cb_userdata(sslContext, (void*)sslPassword);
+	if (!SSL_CTX_use_PrivateKey_file(sslContext, sslKeyFile, SSL_FILETYPE_PEM)) {
+		fprintf(stderr, "Failed to read SSL key file.\n");
+		ERR_print_errors(stderrBio);
+		return NULL;
+	}
+
+	return sslContext;
 }
 
-int sslPasswordCB(char *passwordBuffer, int size, int rwflag, void *data) {
-	char *keyfilePassword;
-	int pwLength;
-
-	keyfilePassword = (char*)data;
-	pwLength = strlen(keyfilePassword);
-
-	if (size < pwLength) {
-  		return 0;
-	}
-	strncpy(passwordBuffer, keyfilePassword, pwLength);
-
-	return pwLength;
-} 
-
-bool sslAccept(int socketFD, SSL **ssl) {
+bool ssl_accept(int socketFD, SSL **ssl, SSL_CTX *sslContext) {
 	BIO *socketBio;
 
 	/* Wrap the socket in a BIO */
@@ -173,31 +164,49 @@ bool sslAccept(int socketFD, SSL **ssl) {
 	return false;
 }
 
-int sslRecv(SSL *ssl, char *buffer, unsigned int maxlength) {
+int ssl_receive(SSL *ssl, char *buffer, unsigned int maxlength) {
 	int status;
 
 	status = SSL_read(ssl, buffer, maxlength);
+#ifdef DEBUG
 	if (status < 0) {
-		BIO_printf(stderrBio, "%s\n", "SSL read failed.");
+		BIO_printf(stderrBio, "SSL read failed.");
 		ERR_print_errors(stderrBio);
 	}
+#endif
 
 	return status;
 }
 
-int sslSend(SSL *ssl, char *buffer, unsigned int length) {
+int ssl_send(SSL *ssl, char *buffer, unsigned int length) {
 	int status;
 	
 	status = SSL_write(ssl, buffer, length);
+#ifdef DEBUG
 	if (status < 0) {
-		BIO_printf(stderrBio, "%s\n", "SSL write failed.");
+		BIO_printf(stderrBio, "SSL write failed.");
 		ERR_print_errors(stderrBio);
 	}
+#endif
 
 	return status;
 }
 
-int sslClose(SSL *ssl) {
+int ssl_close(SSL *ssl) {
 	return SSL_shutdown(ssl);
 }
+
+void ssl_cleanup(SSL_CTX *sslContext) {
+	int i;
+
+	/* Free library resources */
+	SSL_CTX_free(sslContext);
+	BIO_free(stderrBio);
+  
+	/* Destroy locks. */
+	for (i = 0; i < lockCount; i++) {
+		pthread_mutex_destroy(&locks[i]);
+	}
+}
+
 #endif
