@@ -101,22 +101,22 @@ int split_string(const char *str, char **key, char **value, char c) {
 
 /* Check the validity of an URL.
  */
-bool valid_URI(char *URI) {
-	if (*URI != '/') {
+bool valid_uri(char *uri) {
+	if (*uri != '/') {
 		return false;
 	}
 
 	do {
-		if (*(URI + 1) == '.') {
-			if ((*URI == '.') || (*URI == '/')) {
+		if (*(uri + 1) == '.') {
+			if ((*uri == '.') || (*uri == '/')) {
 				return false;
 			}
 		}
-		if ((*URI < 32) || (*URI > 126)) {
+		if ((*uri < 32) || (*uri > 126)) {
 			return false;
 		}
-		URI++;
-	} while (*URI != '\0');
+		uri++;
+	} while (*uri != '\0');
 
 	return true;
 }
@@ -162,14 +162,14 @@ int parse_special_chars(char *line, int length) {
 	return result;
 }
 
-/* Prevent cross-site scripting.
+/* Prevent Command injection.
  */
-int prevent_XSS(char *line) {
+int prevent_CMDi(char *line) {
 	int result = 0;
 
 	if (line != NULL) {
 		while (*line != '\0') {
-			if ((*line == '\"') || (*line == '<') || (*line == '>')) {
+			if ((*line == '|') || (*line == ';') || (*line == '`')) {
 				*line = '_';
 				result += 1;
 			}
@@ -220,6 +220,24 @@ int prevent_SQLi(char *line, int length, char **newline) {
 	return newlength;
 }
 
+/* Prevent cross-site scripting.
+ */
+int prevent_XSS(char *line) {
+	int result = 0;
+
+	if (line != NULL) {
+		while (*line != '\0') {
+			if ((*line == '\"') || (*line == '<') || (*line == '>')) {
+				*line = '_';
+				result += 1;
+			}
+			line++;
+		}
+	}
+
+	return result;
+}
+
 /* Return an errormessage.
  */
 const char *errorstr(int code) {
@@ -274,7 +292,7 @@ const char *errorstr(int code) {
 		{412, "Precondition Failed"},
 		{413, "Request Entity Too Large"},
 /*
-		{414, "URI too long"},
+		{414, "uri too long"},
 		{415, "Unsupported Media Type"},
 */
 		{416, "Requested Range Not Satisfiable"},
