@@ -8,10 +8,11 @@ char *remove_spaces(char *str);
 char *strip_line(char *str);
 char *strlower(char *line);
 int split_string(const char *str, char **key, char **value, char c);
-bool valid_URI(char *URI);
+bool valid_uri(char *uri);
 int parse_special_chars(char *line, int length);
-int prevent_XSS(char *line);
+int prevent_CMDi(char *line);
 int prevent_SQLi(char *line, int length, char **newline);
+int prevent_XSS(char *line);
 const char *errorstr(int code);
 bool decode_base64(char *base64);
 char *file_extension(char *filename);
