@@ -36,58 +36,98 @@ char *hs_eol    = "\r\n";                       //  2
 
 /* Send a char buffer to the client. Traffic throttling is handled here.
  */
-int send_buffer(t_session *session, const char *buffer, int size) {
-	int bytes_send = 0, total_send = 0, can_send, rest;
+static int send_to_client(t_session *session, const char *buffer, int size) {
+	int bytes_sent = 0, total_sent = 0, can_send, rest;
 	time_t new_time;
 
 	/* Send buffer to browser.
 	 */
-	if (size > 0) {
-		if (session->Directory != NULL) {
-			if (session->Directory->SessionSpeed > 0) {
-				session->throttle = session->Directory->SessionSpeed;
-			}
+	if ((buffer == NULL) || (size <= 0)) {
+		return 0;
+	}
+
+	if (session->directory != NULL) {
+		if (session->directory->session_speed > 0) {
+			session->throttle = session->directory->session_speed;
 		}
-		do {
-			rest = size - total_send;
-			if (session->throttle > 0) {
-				do {
-					new_time = time(NULL);
-					if (session->throttle_timer < new_time) {
-						session->bytecounter = 0;
-						session->throttle_timer = new_time;
-					}
-					can_send = session->throttle - session->bytecounter;
-					if (can_send <= 0) {
-						usleep(10000);
-					}
-				} while (can_send <= 0);
-				if (can_send > rest) {
-					can_send = rest;
+	}
+
+	do {
+		rest = size - total_sent;
+		if (session->throttle > 0) {
+			do {
+				new_time = time(NULL);
+				if (session->throttle_timer < new_time) {
+					session->bytecounter = 0;
+					session->throttle_timer = new_time;
 				}
-			} else {
+				can_send = session->throttle - session->bytecounter;
+				if (can_send <= 0) {
+					usleep(10000);
+				}
+			} while (can_send <= 0);
+			if (can_send > rest) {
 				can_send = rest;
 			}
-			
+		} else {
+			can_send = rest;
+		}
+		
 #ifdef HAVE_SSL
-			if (session->via_ssl) {
-				if ((bytes_send = sslSend(session->ssl_data, (char*)(buffer + total_send), can_send)) <= 0) {
-					bytes_send = -1;
-				}
-			} else
+		if (session->binding->use_ssl) {
+			if ((bytes_sent = ssl_send(session->ssl_data, (char*)(buffer + total_sent), can_send)) <= 0) {
+				bytes_sent = -1;
+			}
+		} else
 #endif
-				bytes_send = send(session->client_socket, buffer + total_send, can_send, 0);
+			bytes_sent = send(session->client_socket, buffer + total_sent, can_send, 0);
 
-			if (bytes_send == -1) {
-				close_socket(session);
-				session->keep_alive = false;
+		if (bytes_sent == -1) {
+			close_socket(session);
+			session->keep_alive = false;
+			return -1;
+		}
+		total_sent += bytes_sent;
+		session->bytecounter += bytes_sent;
+	} while (total_sent < size);
+
+	return 0;
+}
+
+/* This function has been added to improve speed by buffering small amounts of data to be sent.
+ */
+int send_buffer(t_session *session, const char *buffer, int size) {
+	if (size > 400) {
+		if (session->output_size > 0) {
+			if (send_to_client(session, session->output_buffer, session->output_size) == -1) {
 				return -1;
 			}
-			total_send += bytes_send;
-			session->bytecounter += bytes_send;
-		} while (total_send < size);
+			session->output_size = 0;
+		}
+		if (send_to_client(session, buffer, size) == -1) {
+			return -1;
+		}
+	} else if (buffer == NULL) {
+		if (session->output_size > 0) {
+			if (send_to_client(session, session->output_buffer, session->output_size) == -1) {
+				return -1;
+			}
+			session->output_size = 0;
+		}
+	} else {
+		if ((session->output_size + size > OUTPUT_BUFFER_SIZE) && (session->output_size > 0)) {
+			if (send_to_client(session, session->output_buffer, session->output_size) == -1) {
+				return -1;
+			}
+			session->output_size = 0;
+		}
+
+		memcpy(session->output_buffer + session->output_size, buffer, size);
+		session->output_size += size;
 	}
 
+	session->bytes_sent += size;
+
 	return 0;
 }
 
@@ -125,11 +165,11 @@ int send_header(t_session *session, int code) {
 			break;
 		} else if (send_buffer(session, hs_server, 8) == -1) {
 			break;
-		} else if (send_buffer(session, session->config->ServerString, strlen(session->config->ServerString)) == -1) {
+		} else if (send_buffer(session, session->config->server_string, strlen(session->config->server_string)) == -1) {
 			break;
 		} else if (send_buffer(session, hs_eol, 2) == -1) {
 			break;
-		} else if ((session->is_CGI_script == false) && (session->URI_is_dir == false)) {
+		} else if ((session->is_cgi_script == false) && (session->uri_is_dir == false)) {
 			if (send_buffer(session, hs_range, 22) == -1) {
 				break;
 			}
@@ -167,66 +207,23 @@ int send_header(t_session *session, int code) {
 
 /* Send a datachunk to the client, used by run_script() in target.c
  */
-int send_chunk(t_session *session, char *chunk, int size) {
+static int send_chunk_to_client(t_session *session, const char *chunk, int size) {
 	char hex[8];
-	bool must_send = false, copy_chunk = false, last_chunk = false;
 
-	if (chunk != NULL) {
-		if (session->chunksize + size <= MAX_CHUNK_SIZE) {
-			if (session->chunkbuffer != NULL) {
-				memcpy(session->chunkbuffer + session->chunksize, chunk, size);
-				session->chunksize += size;
-			} else {
-				copy_chunk = true;
-			}
-		} else {
-			must_send = true;
-			copy_chunk = true;
-		}
-	} else {
-		if (session->HEAD_request) {
-			if (session->chunkbuffer != NULL) {
-				free(session->chunkbuffer);
-				session->chunkbuffer = NULL;
-				session->chunksize = 0;
-			}
-		} else {
-			must_send = true;
-			last_chunk = true;
-		}
-	}
-
-	if (must_send && (session->chunksize > 0)) {
-		if (session->keep_alive) {
-			if (sprintf(hex, "%x\r\n", session->chunksize) >= 0) {
-				if (send_buffer(session, hex, strlen(hex)) == -1) {
-					return -1;
-				}
-			} else {
-				session->keep_alive = false;
-			}
-		}
-		if (send_buffer(session, session->chunkbuffer, session->chunksize) == -1) {
+	if (session->keep_alive) {
+		if (sprintf(hex, "%x\r\n", size) < 0) {
+			return -1;
+		} else if (send_to_client(session, hex, strlen(hex)) == -1) {
 			return -1;
-		} else if (session->keep_alive) {
-			if (send_buffer(session, "\r\n", 2) == -1) {
-				return -1;
-			}
 		}
-		free(session->chunkbuffer);
-		session->chunkbuffer = NULL;
-		session->chunksize = 0;
 	}
 
-	if (last_chunk && session->keep_alive) {
-		send_buffer(session, "0\r\n\r\n", 5);
+	if (send_to_client(session, chunk, size) == -1) {
+		return -1;
 	}
-
-	if (copy_chunk) {
-		if ((session->chunkbuffer = (char*)malloc(MAX_CHUNK_SIZE)) != NULL) {
-			memcpy(session->chunkbuffer, chunk, size);
-			session->chunksize = size;
-		} else {
+	
+	if (session->keep_alive) {
+		if (send_to_client(session, "\r\n", 2) == -1) {
 			return -1;
 		}
 	}
@@ -234,15 +231,54 @@ int send_chunk(t_session *session, char *chunk, int size) {
 	return 0;
 }
 
+int send_chunk(t_session *session, const char *chunk, int size) {
+	if (size > 400) {
+		if (session->output_size > 0) {
+			if (send_chunk_to_client(session, session->output_buffer, session->output_size) == -1) {
+				return -1;
+			}
+			session->output_size = 0;
+		}
+		if (send_chunk_to_client(session, chunk, size) == -1) {
+			return -1;
+		}
+	} else if (chunk == NULL) {
+		if (session->output_size > 0) {
+			if (send_chunk_to_client(session, session->output_buffer, session->output_size) == -1) {
+				return -1;
+			}
+			session->output_size = 0;
+		}
+		if (send_to_client(session, "0\r\n\r\n", 5) == -1) {
+			return -1;
+		}
+	} else {
+		if ((session->output_size + size > OUTPUT_BUFFER_SIZE) && (session->output_size > 0)) {
+			if (send_chunk_to_client(session, session->output_buffer, session->output_size) == -1) {
+				return -1;
+			}
+			session->output_size = 0;
+		}
+
+		memcpy(session->output_buffer + session->output_size, chunk, size);
+		session->output_size += size;
+	}
+
+	session->bytes_sent += size;
+
+	return 0;
+}
+
 /* Send a HTTP code to the client. Used in case of an error.
  */
 int send_code(t_session *session, int code) {
-	int retval = -1, binded_port;
+	int retval = -1, default_port;
 	char ecode[4], len[8], port[16];
 	const char *emesg;
 
 	/* Send simple HTTP error message.
 	 */
+	session->mimetype = NULL;
 	if (send_header(session, code) == -1) {
 		return retval;
 	}
@@ -253,7 +289,7 @@ int send_code(t_session *session, int code) {
 				break;
 			}
 #ifdef HAVE_SSL
-			if (session->via_ssl) {
+			if (session->binding->use_ssl || (session->cause_of_301 == require_ssl)) {
 				if (send_buffer(session, "s", 1) == -1) {
 					break;
 				}
@@ -263,28 +299,33 @@ int send_code(t_session *session, int code) {
 				break;
 			}
 
-			if (send_buffer(session, *(session->host->Hostname.item), strlen(*(session->host->Hostname.item))) == -1) {
+			if (send_buffer(session, *(session->host->hostname.item), strlen(*(session->host->hostname.item))) == -1) {
 				break;
 			}
 
 #ifdef HAVE_SSL
-			if (session->via_ssl) {
-				binded_port = 443;
+			if (session->binding->use_ssl) {
+				default_port = 443;
 			} else 
 #endif
-				binded_port = 80;
+				default_port = 80;
 
-			if (session->binding->port != binded_port) {
+			if (session->binding->port != default_port) {
 				sprintf(port, ":%d", session->binding->port);
 				if (send_buffer(session, port, strlen(port)) == -1) {
 					break;
 				}
 			}
-			if (send_buffer(session, session->URI, strlen(session->URI)) == -1) {
-				break;
-			} else if (send_buffer(session, "/", 1) == -1) {
+			if (send_buffer(session, session->uri, strlen(session->uri)) == -1) {
 				break;
 			}
+#ifdef HAVE_SSL
+			if (session->cause_of_301 == missing_slash)
+#endif
+				if (send_buffer(session, "/", 1) == -1) {
+					break;
+				}
+
 			if (session->vars != NULL) {
 				if (send_buffer(session, "?", 1) == -1) {
 					break;
@@ -311,7 +352,7 @@ int send_code(t_session *session, int code) {
 			retval = -1;
 			sprintf(ecode, "%d", code);
 			emesg = errorstr(code);
-			sprintf(len, "%d", 2 * strlen(emesg) + 353);
+			sprintf(len, "%d", 2 * strlen(emesg) + 291);
 
 			if (send_buffer(session, "Content-Length: ", 16) == -1) {
 				break;
@@ -323,32 +364,32 @@ int send_code(t_session *session, int code) {
 				break;
 			} else if (send_buffer(session, "text/html\r\n\r\n", 13) == -1) {
 				break;
-			} else if (session->HEAD_request) {
+			} else if (session->head_request) {
 				retval = 0;
 				break;
 			} else if (send_buffer(session, "<html><head><title>", 19) == -1) {
 				break;
 			} else if (send_buffer(session, ecode, 3) == -1) {
 				break;
-			} else if (send_buffer(session, ": ", 2) == -1) {
+			} else if (send_buffer(session, " - ", 3) == -1) {
 				break;
 			} else if (send_buffer(session, emesg, strlen(emesg)) == -1) {
 				break;
-			} else if (send_buffer(session,	"</title>\n" // 9
-											"<style type=\"text/css\">" // 23
-											"BODY { color:#ffff00 ; background-color:#000040 } " // 50
-											"H1 { font-size:50px ; margin:0px ; font-weight:normal } " // 56
-											"TD { font-family: arial ; font-size:20px ; text-align:center ; vertical-align:middle }" //86
-											"</style></head>\n" // 16
-											"<body><table width=100% height=100%><tr><td><h1>", 288) == -1) { // 48
+			} else if (send_buffer(session,	"</title><style type=\"text/css\">\n" // 32
+											"<!--\n" // 5
+											"BODY { color:#ffffff ; background-color:#00000a }\n" // 50
+											"DIV { font-family:sans-serif ; font-size:30px ; letter-spacing:20px ; " // 70
+											"text-align:center ; position:relative ; top:250px }\n" // 52
+											"--></style></head>\n" // 19
+											"<body><div>", 239) == -1) { // 11
 				break;
 			} else if (send_buffer(session, ecode, 3) == -1) {
 				break;
-			} else if (send_buffer(session, "</h1>", 5) == -1) {
+			} else if (send_buffer(session, " - ", 3) == -1) {
 				break;
 			} else if (send_buffer(session, emesg, strlen(emesg)) == -1) {
 				break;
-			} else if (send_buffer(session, "</td></tr></table></body></html>\n", 33) == -1) {
+			} else if (send_buffer(session, "</div></body></html>\n", 21) == -1) {
 				break;
 			}
 
@@ -367,10 +408,10 @@ int send_basic_auth(t_session *session) {
 	do {
 		if (send_buffer(session, "WWW-Authenticate: Basic", 23) == -1) {
 			break;
-		} else if (session->host->LoginMessage != NULL) {
+		} else if (session->host->login_message != NULL) {
 			if (send_buffer(session, " realm=\"", 8) == -1) {
 				break;
-			} else if (send_buffer(session, session->host->LoginMessage, strlen(session->host->LoginMessage)) == -1) {
+			} else if (send_buffer(session, session->host->login_message, strlen(session->host->login_message)) == -1) {
 				break;
 			} else if (send_buffer(session, "\"", 1) == -1) {
 				break;
