@@ -5,7 +5,7 @@
 
 int send_buffer(t_session *session, const char *buffer, int size);
 int send_header(t_session *session, int code);
-int send_chunk(t_session *session, char *chunk, int size);
+int send_chunk(t_session *session, const char *chunk, int size);
 int send_code(t_session *session, int code);
 
 int send_basic_auth(t_session *session);
