@@ -3,10 +3,11 @@
 
 #include "session.h"
 
-#define rr_SQL_INJECTION -100
+#define rr_SQL_INJECTION -50
+#define rr_CMD_INJECTION -51
 
 int send_file(t_session *session);
-int run_script(t_session *session, int code);
+int run_cgi_program(t_session *session);
 int show_index(t_session *session);
 int handle_options_request(t_session *session);
 int handle_trace_request(t_session *session);
